"use strict";

const React = require("react");
const classnames = require("classnames");

const Spinner = require("./spinner");
const {Form, Input, SubmitButton, FormSpinner} = require("./form");
const MainView = require("./main-view");
const useModule = require("../plumbing/use-module");

function AddFriendForm() {
	return (
		<Form module="uiAddFriend" className="horizontal">
			<Input name="username" placeholder="Username" />
			<SubmitButton>Add friend</SubmitButton>
			<FormSpinner />
		</Form>
	);
}

function InviteFriendForm() {
	return (
		<Form module="uiInviteFriend" className="horizontal">
			<Input name="note" placeholder="Note (optional)" />
			<SubmitButton>Create new invite code</SubmitButton>
			<FormSpinner />
		</Form>
	);
}

function InviteDeleteButton({invite}) {
	let {dispatch} = useModule("invites");

	function handleDelete(id) {
		dispatch({
			type: "delete",
			inviteId: id
		});
	}

	return (<>
		<button disabled={invite.beingDeleted} onClick={() => handleDelete(invite.id)}>
			Delete code
		</button>
		{invite.beingDeleted
			? <Spinner />
			: null
		}
	</>);
}

function InvitationsTable() {
	let {state} = useModule("invites");
	let {invites} = state;

	return (
		<table className="invites">
			<thead>
				<tr>
					<th>Invite code</th>
					<th>Note</th>
					<th></th>
				</tr>
			</thead>
			<tbody>
				{invites.toList().map((invite) => {
					return (
						<tr key={invite.id} className={classnames({consumed: invite.consumed})}>
							<td>{invite.code}</td>
							<td>{invite.note}</td>
							<td className="status">
								{(invite.consumed)
									? "Code has been used"
									: <InviteDeleteButton invite={invite} />
								}
							</td>
						</tr>
					);
				})}
			</tbody>
		</table>
	);
}

function ViewContents() {
	/* FIXME: Quick flash-of-missing-form occurs, presumably due to the nested component hack in with InnerForm. */
	return (
		<MainView.Contents>
			<h2>Add a friend</h2>
			<p>If your friend already has an account, simply enter their username below and press the button! They'll immediately show up in your friends list, but they'll need to accept your friend request before you can talk to them.</p>

			<AddFriendForm />

			<h2>Invite a friend</h2>
			<p>If your friend does not have an account yet, you can create an invite code for them below - they can then use this invite code to create an account. Once they do, you'll be added to each other's friend lists automatically.</p>
			<p>Optionally, you may add a custom note for the invite code; this allows you to keep track of who you gave an invite code.</p>
			<p><em>Invites are unlimited, but we keep track of who invited whom, for spam prevention reasons. Please only give invites to people you know; accounts that are used to provide automated public invites may be disabled.</em></p>

			<InviteFriendForm />
			<InvitationsTable />
		</MainView.Contents>
	);
};

module.exports = function AddFriendView() {
	return (
		<MainView.Container className="meta">
			<MainView.Header>
				<div className="headerText">
					Add or invite a friend
				</div>
			</MainView.Header>
			<ViewContents />
		</MainView.Container>
	);
};