"use strict";

const React = require("react");
const classnames = require("classnames");
const defaultValue = require("default-value");

const Frame = require("./frame");
const StatusBall = require("./status-ball");
const useStore = require("../plumbing/use-store");
const useModule = require("../plumbing/use-module");

function FriendItem({children, className, selected, onClick}) {
	return (
		<div className={classnames("friend", className, {selected: selected})} onClick={onClick}>
			{children}
		</div>
	);
}

function Friend({friend, selected, onClick}) {
	return (
		<FriendItem selected={selected} onClick={onClick}>
			<StatusBall status={friend.status} statusVisible={friend.statusVisible} />
			{defaultValue(friend.displayName, friend.username)}
		</FriendItem>
	);
}

function FriendMeta({children, onClick, selected}) {
	return (
		<FriendItem className="meta" onClick={onClick} selected={selected}>
			{children}
		</FriendItem>
	);
}

module.exports = function FriendList() {
	let {state, dispatch} = useStore();
	let {selectedTab, selectedTabType} = state;
	let selectedAddFriendTab = (selectedTabType=== "meta" && selectedTab === "addFriend");

	let {friends} = useModule("friends").state;

	function selectConversationTab(username) {
		dispatch({
			type: "selectedConversationTab",
			username: username
		});
	}

	function selectAddFriendTab() {
		dispatch({ type: "selectedAddFriendTab" });
	}
	
	return (
		<Frame className="friendList">
			{friends.toList().toArray().map((friend) => {
				let selected = (selectedTabType === "conversation" && selectedTab === friend.username);

				return <Friend key={friend.username} friend={friend} selected={selected} onClick={() => selectConversationTab(friend.username)} />;
			})}
			<FriendMeta selected={selectedAddFriendTab} onClick={selectAddFriendTab}>
				Add or invite a friend...
			</FriendMeta>
		</Frame>
	);
};

// let ConnectedFriendList = reactRedux.connect((state) => {
// 	return {
// 		/* TODO: Memoize */
// 		friends: state.friends.toList().toArray(), /* ?! Why can't a Map be directly converted to an array of values... */
// 		selectedTabType: state.selectedTabType,
// 		selectedTab: state.selectedTab
// 	};
// }, {
// 	selectAddFriendTab: () => {
// 		return {
// 			type: "selectedAddFriendTab"
// 		};
// 	},
// 	selectConversationTab: (username) => {
// 		return {
// 			type: "selectedConversationTab",
// 			username: username
// 		};
// 	}
// })(function FriendList({friends, selectedTabType, selectedTab, selectAddFriendTab, selectConversationTab}) {
// 	/* FIXME: Figure out a nicer pattern for dealing with tab selection... this is a mess. */
// 	let selectedAddFriendTab = (selectedTabType=== "meta" && selectedTab === "addFriend");

// 	return (
// 		<Frame className="friendList">
// 			{friends.map((friend) => {
// 				let selected = (selectedTabType === "conversation" && selectedTab === friend.username);

// 				return <Friend key={friend.username} friend={friend} selected={selected} onClick={() => selectConversationTab(friend.username)} />;
// 			})}
// 			<FriendMeta selected={selectedAddFriendTab} onClick={selectAddFriendTab}>
// 				Add or invite a friend...
// 			</FriendMeta>
// 		</Frame>
// 	);
// });

// module.exports = ConnectedFriendList;