"use strict";

const React = require("react");
const reactRedux = require("react-redux");

const ProfileCard = require("./profile-card");
const FriendList = require("./friend-list");
const ConversationView = require("./conversation-view");
const AddFriendView = require("./add-friend-view");
const match = require("../match");
const data = require("../plumbing/data");

const mockInit = require("../init");

let MainViewArea = reactRedux.connect((state) => {
	return {
		selectedTabType: state.selectedTabType,
		selectedTab: state.selectedTab
	};
})(function MainViewArea({selectedTabType, selectedTab}) {
	/* TODO: Log invalidly called view configurations */
	
	return match(selectedTabType, {
		conversation: () => {
			return <ConversationView />;
		},
		meta: () => {
			return match(selectedTab, {
				addFriend: () => {
					return <AddFriendView />;
				},
				_: () => {
					<div className="error">ERROR: Unknown view</div>
				}
			})
		},
		_: () => {
			if (selectedTabType == null) {
				return <div></div>;
			} else {
				return <div className="error">ERROR: Unknown view type: {selectedTabType}</div>;
			}
		}
	});
});

module.exports = function App() {
	let [store, setStore] = React.useState();

	React.useEffect(() => {
		let newStore = data.createStore();

		mockInit(newStore);

		setStore(newStore);
	}, []);

	if (store != null) {
		return (
			<reactRedux.Provider store={store}>
				<div className="app">
					<div className="sidebar">
						<ProfileCard />
						<FriendList />
					</div>
					<MainViewArea />
				</div>
			</reactRedux.Provider>
		);
	} else {
		return null;
	}
};