"use strict";

const React = require("react");
const reactRedux = require("react-redux");
const defaultValue = require("default-value");

const Frame = require("./frame");
const StatusBall = require("./status-ball");
const Picker = require("./picker");
const humanReadableStatus = require("../human-readable-status");

function StatusPicker({status}) {
	return (
		<Picker className="statusPicker">
			<StatusBall status={status} statusVisible={true} />
			{humanReadableStatus(status)}
		</Picker>
	);
}

function ProfileCard({status, displayName, username, statusMessage}) {
	return (
		<Frame className="profileCard">
			<div className="profileInformation">
				<div className="displayName">
					{displayName}
				</div>
				<div className="username">
					{username}
				</div>
			</div>
			<StatusPicker status={status} />
			<div className="statusMessage">
				<input type="text" placeholder="Enter a status message here..." value={defaultValue(statusMessage, undefined)} />
			</div>
		</Frame>
	);
}

let ConnectedProfileCard = reactRedux.connect((state) => {
	return {
		status: state.status,
		displayName: state.displayName,
		username: state.username,
		statusMessage: state.statusMessage
	};
})(ProfileCard);

module.exports = ConnectedProfileCard;