"use strict";

const Promise = require("bluebird");

let addFriendLogic = createLogic({
    type: "uiAddFriend",
    processOptions: {
        dispatchReturn: true,
        successType: "uiAddFriendDBStored",
        failType: "uiAddFriendError"
    },
    validate: ({getState, action}, allow, reject) => {
        /* QUESTION: Should we add the new friend to the list locally before acknowledgement from the server? What if the server then returns an error concerning the addition, should it be removed again? */
        if (action.username.length === 0) {
            /* Error: you must enter a username */
            reject({
                type: "uiAddFriendError",
                error: true,
                errorType: "noUsernameSpecified"
            });
        } else if (getState().friends.has(action.username)) {
            /* Error: that user is already in your friends list */
            reject({
                type: "uiAddFriendError",
                error: true,
                errorType: "friendAlreadyInList"
            });
        } else {
            allow(action);
        }
    },
    process: ({action, api}, dispatch, done) => {
        dispatch({
            type: "uiAddFriendProcessing"
        });

        return api.addFriend({
            username: action.username
        });
    }
});