"use strict";

const types = require("../plumbing/types");

module.exports = {
	name: "uiAddFriend",
	mixins: [
		require("./mixins/form")({
			validate: ({globalState, formData, allow, reject}) => {
				let {username} = formData;

				if (username == null) {
					reject({
						errorType: "noUsernameSpecified",
						errorMessage: "You must specify a username.",
						invalidFields: ["username"]
					});
				} else if (globalState.modules.friends.friends.has(username)) {
					reject({
						errorType: "friendAlreadyInList",
						/* TODO: Make error message clearer, maybe include display name of existing entry? */
						errorMessage: "There's already a user with that username in your friends list.",
						invalidFields: ["username"]
					});
				} else {
					allow();
				}
			},
			process: ({formData, api}) => {
				return api.addFriend({
					username: formData.username
				});
			},
			success: ({result}) => {
				let friend = types.Friend(result.friend);

				return [{
					type: "@friends_added",
					friend: friend
				}, {
					type: "@friends_addedDbStored",
					username: friend.username,
					optional: true
				}, {
					type: "@selectedConversationTab",
					username: friend.username
				}];
			}
		})
	]
};