"use strict";

const immutable = require("immutable");

function applyMessageDefaults(message) {
	if (message.direction === "received") {
		return message.merge({
			delivered: true,
			deliveredDBStored: true,
			dbStored: true
		});
	} else {
		return message;
	}
}

module.exports = {
	name: "messages",
	state: {
		messages: immutable.Map()
	},
	stateTransformations: {
		add: (state, action) => {
			let {message} = action;

			return state.updateIn(["messages"], (messages) => {
				return messages.set(message.id, applyMessageDefaults(message));
			});
		},
		addFailed: (state, action) => {
			/* FIXME: Eventually, attempt to re-send the message? */
			console.log("failed:", action);
			return state;
		},
		dbStored: (state, action) => {
			let {messageId} = action;

			return state.setIn(["messages", messageId, "dbStored"], true);
		}
	},
	logicHandlers: {
		add: {
			errorType: "addFailed",
			validate: ({action, reject, allow}) => {
				let {message} = action;

				if (message.id == null) {
					reject({ errorMessage: "Message must contain an ID" });
				} else {
					allow();
				}
			},
			process: ({action, dispatch, api}) => {
				let {message} = action;

				dispatch({
					type: "@friends_addedMessageToConversation",
					message: message,
					username: (message.direction === "sent") ? message.recipient : message.sender
				});

				if (!message.dbStored) {
					return api.sendMessage({message});
				}
			},
			success: ({action}) => {
				let {message} = action;

				if (message.direction === "sent") {
					return {
						type: "dbStored",
						messageId: message.id
					};
				}
			},
			failure: ({error}) => ({errorMessage: error.message})
		}
	}
};