"use strict";

const createLogic = require("./create-logic");
const types = require("./types");

module.exports = createLogic({
	type: "uiAddFriend",
	errorType: "uiAddFriendError",
	validate: ({getState, action}, allow, reject) => {
		if (action.username.length === 0) {
			/* Error: you must enter a username */
			reject({
				errorType: "noUsernameSpecified",
				errorMessage: "You must specify a username."
			});
		} else if (getState().friends.has(action.username)) {
			/* Error: that user is already in your friends list */
			reject({
				errorType: "friendAlreadyInList",
				errorMessage: "There's already a user with that username in your friends list."
			});
		} else {
			allow();
		}
	},
	process: ({action, api}, dispatch) => {
		dispatch({
			type: "uiAddFriendProcessing"
		});
		
		return api.addFriend({
			username: action.username
		});
	},
	success: (result) => {
		let friend = types.Friend(result.friend);

		return [{
			type: "uiAddFriendComplete",
			friend: friend
		}, {
			type: "friendAdded",
			friend: friend
		}, {
			type: "friendAddedDbStored",
			username: friend.username
		}];
	},
	failure: (error) => {
		/* FIXME: Error filtering to distinguish bugs from end-user errors? */
		return {
			errorMessage: error.message
		};
	}
});