"use strict";

const immutable = require("immutable");

let Friend = immutable.Record({
	username: null, // string
	displayName: null, // string
	statusVisible: false, // boolean
	status: null, // {"online", "away", "offline"}
	statusMessage: null, // string
	isTyping: false, // boolean
	pausedTyping: false, // boolean
	messageIds: immutable.List(), // (messages) id
	messageGroups: immutable.List(), // MessageGroup
	mutual: false,
	scrollTop: null,
	scrollLocked: null
});

let FriendRequest = immutable.Record({
	/* TODO: Show more profile information here in the future, such as the display name? */
	username: null, // string
	message: null, // string
	beingAccepted: false, // boolean
	beingRejected: false, // boolean
});

let UiFormState = immutable.Record({
	processing: false, // boolean
	errorMessage: null, // string
	invalidFields: immutable.Set(), // string
	resetCounter: 0, // number
	wasFocused: false, // boolean
	lastValues: null, // object (key -> value)
});

let Message = immutable.Record({
	id: null, // string
	direction: null, // {"sent", "received"}
	sender: null, // null | (friends) username
	recipient: null, // null | (friends) username
	timestamp: null, // Date
	body: null, // string
	dbStored: false, // boolean
});

let MessageGroup = immutable.Record({
	id: null, // string
	direction: null, // {"sent", "received"}
	sender: null, // null | (friends) username
	recipient: null, // null | (friends) username
	firstTimestamp: null, // Date
	messageIds: immutable.List() // (messages) id
})

let Invite = immutable.Record({
	id: null, // string
	note: null, // string
	code: null, // string
	consumed: false, // boolean
	friendId: null, // (friends) id -- note: can become a non-existent reference over time!
	beingDeleted: false
});

module.exports = {
	Friend, FriendRequest, UiFormState, Message, MessageGroup, Invite
};