"use strict";

const React = require("react");
const reactRedux = require("react-redux");

const scopedKey = require("./modules/scoped-key");

module.exports = function useModule(moduleName) {
	let {store} = React.useContext(reactRedux.ReactReduxContext);

	function getState() {
		return store.getState().modules[moduleName];
	}

	let moduleState = getState();

	if (moduleState != null) {
		return {
			state: moduleState,
			getState: getState,
			dispatch: (action) => {
				store.dispatch(Object.assign({}, action, {
					type: scopedKey(moduleName, action.type)
				}));
			}
		};
	} else {
		throw new Error(`The specified module '${moduleName}' does not seem to exist`);
	}
};