"use strict";

const React = require("react");
const defaultValue = require("default-value");

module.exports = function useLockableScroll(options) {
	let {onScroll} = options;
	let scrollThreshold = defaultValue(options.threshold, 10);
	let upstreamScrollTop = options.scrollTop;
	let upstreamLocked = options.locked;
	let upstreamKey = options.scrollKey;

	let [localLocked, setLocalLocked] = React.useState(false);
	let [ref, setRef] = React.useState();
	let blockScrollEcho = React.useRef(false);

	let locked = defaultValue(upstreamLocked, localLocked);

	React.useLayoutEffect(() => {
		// console.log("doing scroll", locked, upstreamScrollTop, content);
		
		console.log("doing scroll; upstream locked:", upstreamLocked);
		

		if (ref != null) {
			blockScrollEcho.current = true;

			if (locked === true) {
				ref.scrollTop = ref.scrollHeight - ref.offsetHeight;
			} else if (upstreamScrollTop != null) {
				ref.scrollTop = upstreamScrollTop;
			}
		}
	}, [upstreamKey]);

	React.useEffect(() => {
		/* TODO: Keep this from being called so often, even when not changing between views. */
		if (ref != null) {
			function handleScroll(_event) {
				if (blockScrollEcho.current === true) {
					/* This occurs immediately after switching views, and restoring the scroll position. There's no point in handling the scroll event resulting from our own code, so we skip it entirely. */
					console.log("skipping scroll");
					
					blockScrollEcho.current = false;
				} else {
					if (ref != null) {
						let shouldBeLocked = ref.scrollTop > (ref.scrollHeight - ref.offsetHeight - scrollThreshold);
	
						if (onScroll != null) {
							onScroll(shouldBeLocked, ref.scrollTop);
						}
						
						setLocalLocked(shouldBeLocked);
					}
				}
			}

			ref.addEventListener("scroll", handleScroll);

			return () => {
				ref.removeEventListener("scroll", handleScroll);
			};
		}
	}, [ref, onScroll]);

	return [localLocked, setRef];
};