"use strict";

const React = require("react");
const dateFns = require("date-fns");

function millisecondsHavePassed(date, milliseconds) {
	return dateFns.isBefore(dateFns.addMilliseconds(date, milliseconds), Date.now());
}

module.exports = function useTimeout(milliseconds, baseDate = Date.now()) {
	let [referenceDate, _setReferenceDate] = React.useState(baseDate);
	let [timeoutHit, setTimeoutHit] = React.useState(false);

	React.useEffect(() => {
		let timeout;

		if (millisecondsHavePassed(referenceDate, milliseconds)) {
			setTimeoutHit(true);
		} else {
			let timeSinceReferenceDate = dateFns.differenceInMilliseconds(Date.now(), referenceDate);
			let millisecondsLeft = milliseconds - timeSinceReferenceDate;

			timeout = setTimeout(() => {
				setTimeoutHit(true);
				timeout = null;
			}, millisecondsLeft);
		}

		return () => {
			if (timeout != null) {
				clearTimeout(timeout);
			}
		}
	}, []);

	return timeoutHit;
};